/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;


import net.sf.basedb.core.BioMaterial;
import net.sf.basedb.core.BioMaterialEventSource;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.util.formatter.Formatter;

/**
 	Formatter to handle {@link BioMaterialEventSource} items.
 	It will print the name + used quantity from the source item.
 	
  	@author Nicklas
  	@version 3.1
  	@base.modified $Dat$
 */
public class BioMaterialEventSourceFormatter 
	implements Formatter<BioMaterialEventSource>
{
	private String nullValue = "";

	/**
		Create a new formatter. Null values are formatted as an empty string.
	*/
	public BioMaterialEventSourceFormatter()
	{}
	
	/**
		Create a new formatter.
		@param nullValue The string to return if a null item is 
			passed to {@link #format(BioMaterialEventSource)}
	*/
	public BioMaterialEventSourceFormatter(String nullValue)
	{
		this.nullValue = nullValue;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(BioMaterialEventSource eventSource)
	{
		if (eventSource == null) return nullValue;
		try
		{
			BioMaterial source = eventSource.getBioMaterial();
			Float usedQuantity = eventSource.getUsedQuantity();
			return source.getName() + (usedQuantity != null ? " (" + usedQuantity + "µg)" : "");
		}
		catch (PermissionDeniedException ex)
		{}
		return nullValue;
	}

	/**
		@throws UnsupportedOperationException Always
	*/
	@Override
	public BioMaterialEventSource parseString(String value)
	{
		throw new UnsupportedOperationException("parseString: " + value);
	}
	// -------------------------------------------

}
