/**
	$Id $

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import java.util.Map;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Metadata.PropertyPath;

/**	
	Data loader implementation that load values from a map
	collection. The key of the map must be specified when
	constructing the data loader. 
	
	@author Nicklas
	@since 3.8
*/
public class MapValueLoader
	implements DataLoader<BasicItem>
{
	
	private final PropertyPath<BasicItem, Map<?, ?>> propertyPath;
	private final Object mapKey;

	/**
		Load values from a map.
		
		@param propertyPath Property path to the map collection
		@param mapKey The key to use when loading the value
	*/
	public MapValueLoader(PropertyPath<BasicItem, Map<?, ?>> propertyPath, Object mapKey)
	{
		this.propertyPath = propertyPath;
		this.mapKey = mapKey;
	}
	
	@Override
	public Object getData(DbControl dc, ExportedProperty exportedProperty, BasicItem item) 
	{
		Object value = null;
		Map<?, ?> map = propertyPath.getValue(dc, item);
		if (map != null)
		{
			value = map.get(mapKey);
		}
		return value;
	}

}
