/**
	$Id: BasicItemToIdTransformer.java 4712 2008-12-19 13:55:13Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.collections;

import java.util.Collection;

import net.sf.basedb.core.BasicItem;

/**
	Collection transformer implementation that extracts the ID from
	{@link BasicItem}:s.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2008-12-19 14:55:13 +0100 (fr, 19 dec 2008) $
*/
public class BasicItemToIdTransformer
	implements CollectionTransformer<BasicItem, Integer>
{

	public BasicItemToIdTransformer()
	{}
	
	/*
		From the CollectionTransformer interface
		----------------------------------------
	*/
	/**
		Put the ID of each BasicItem in the source collection into the 
		destination collection.
		@return The number of items added to the destination
	*/
	@Override
	public int transform(Collection<? extends BasicItem> source, Collection<? super Integer> destination)
	{
		int added = 0;
		for (BasicItem item : source)
		{
			if (destination.add(item.getId())) added++;
		}
		return added;
	}
	// --------------------------------------------
}
