/**
	$Id $

	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import net.sf.basedb.util.Values;

/**
 	Formats a number for output on a web page adapting thwe number of 
 	decimals depending on the order of magnitude of the value. The precision
 	value is defined to be the number of decimals to use for values 
 	between 1 and 10 (not inclusive). The number of decimals are then
 	increase or decreased for each order of magnitude that the value
 	get smaller or larger.

	@author nicklas
	@since 3.2.2
	@base.modified $Date$
*/
public class AdaptiveNumberFormatter
	extends NumberFormatter
{

	private final int precision;
	/**
		Create a new number formatter.
		@param precision The number of decimals to display for values 
			between 1 and 10 (for lower and higher values the number of decimals is
			adjusted), or a negative value to display all decimals
		@param doubleValues If the {@link #parseString(String)} should return double 
			or float values
		@see Values#formatNumber(Float, int)
	*/
	public AdaptiveNumberFormatter(int precision, boolean doubleValues)
	{
		super(precision, doubleValues);
		this.precision = precision;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(Number value)
	{
		if (value == null) return "";
		int numDecimals = precision;
		if (precision >= 0)
		{
			int orderOfMagnitude = (int)Math.floor(Math.log10(Math.abs(value.doubleValue()))); // =0 for values between 1-10
			if (orderOfMagnitude < 0) orderOfMagnitude++; // One decimal less for value between 0 and 1
			numDecimals -= orderOfMagnitude;
			if (numDecimals < 0) numDecimals = 0;
			if (numDecimals > 7) numDecimals = 7;
		}
		return Values.formatNumber(value.floatValue(), numDecimals);
	}
	// -------------------------------------------

}
