/*
	$Id: JepRestrictionFunction.java 5019 2009-06-26 07:37:02Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.query.Restriction;

import org.nfunk.jep.Node;

/**
	This interface should be implemented by JEP functions
	that can also be converted into a {@link Restriction} that
	can be used in queries.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2009-06-26 09:37:02 +0200 (fr, 26 jun 2009) $
	@see Jep#formulaToRestriction(String, JepFunction[])
*/
public interface JepRestrictionFunction
	extends JepFunction
{

	/**
		Convert this function to a {@link Restriction} that can be used
		in a query. The implementation must check that the node contains
		the correct number of children (ie. arguments to the function)
		and convert the arguments to whatever is suitable for the restriction.
		Example from the {@link InReporterListFunction}: <code>inList(list-id)</code>
		<pre class="code">
public Restriction toRestriction(Node node)
{
  int numChildren = node.jjtGetNumChildren();
  if (numChildren != 1)
  {
    throw new BaseException("Invalid number of expressions for 'inList' function: " + numChildren);
  }
  int reporterListId = Jep.nodeToInt(node.jjtGetChild(0));
  ReporterList list = ReporterList.getById(dc, reporterListId);
  return Dynamic.isPartOf(list);
}
</pre>

		@param node The node representing this function
		@return A <code>Restriction</code> object
	*/
	public Restriction toRestriction(Node node);
}
