/**
	$Id: AnyToAnyNameGenerator.java 5500 2010-11-18 07:26:30Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.node;

import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.util.overview.Node;

/**
	Name generator for any-to-any links. We construct the
	name by taking the name of the link + the name of the
	linked item.

	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-18 08:26:30 +0100 (to, 18 nov 2010) $
*/
public class AnyToAnyNameGenerator
	implements NodeNameGenerator<AnyToAny>
{
	public AnyToAnyNameGenerator()
	{}

	/*
		From the NodeNameGenerator interface
		------------------------------------
	*/
	@Override
	public String getNodeName(AnyToAny item, Node parentNode)
	{
		return "any." + item.getId();
	}

	@Override
	public String getNodeTitle(AnyToAny item, Node parentNode)
	{
		String title = item.getName() + ": ";
		try
		{
			BasicItem to = item.getTo();
			if (to instanceof Nameable)
			{
				title += ((Nameable)to).getName();
			}
			else
			{
				title += to.toString();
			}
		}
		catch (PermissionDeniedException ex)
		{
			title += "denied";
		}
		catch (ItemNotFoundException ex)
		{
			title += "missing";
		}
		return title;
	}
	@Override
	public String getDeniedNodeName(Node parentNode)
	{
		return "any";
	}

	@Override
	public String getDeniedNodeTitle(Node parentNode)
	{
		return "Any-to-any: denied";
	}

	@Override
	public String getMissingNodeName(Node parentNode)
	{
		return "any";
	}

	@Override
	public String getMissingNodeTitle(Node parentNode)
	{
		return "Any-to-any: missing";
	}
	// --------------------------------------
}