/*
	$Id: TestExtensions.java 7190 2016-08-29 08:45:59Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.test.extension.IActionButton;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.Action;
import net.sf.basedb.util.extensions.ActionFactory;
import net.sf.basedb.util.extensions.ClientContext;
import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;
import net.sf.basedb.util.extensions.ExtensionsInvoker;
import net.sf.basedb.util.extensions.InvokationContext;
import net.sf.basedb.util.extensions.Registry;
import net.sf.basedb.util.extensions.Renderer;
import net.sf.basedb.util.extensions.RendererFactory;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.processor.RegisterExtensionsProcessor;
import net.sf.basedb.util.extensions.xml.PathConverter;
import net.sf.basedb.util.extensions.xml.PathSetter;
import net.sf.basedb.util.extensions.xml.VariableConverter;
import net.sf.basedb.util.extensions.xml.VariableSetter;
import net.sf.basedb.util.extensions.xml.XmlLoader;



public class TestExtensions 
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}	
	
	static boolean test_all()
	{
		write("++Testing extensions");

		Registry registry = new Registry();
		ExtensionsManager manager = new ExtensionsManager(registry, null);

		test_load_extension_points_from_xml(registry, "data/test.extensions.xml", 1);
		test_load_extensions_from_xml(registry, "data/test.extensions.xml", 5);
		test_load_extensions_from_jar(manager, 3, "ExtensionOne.jar", "ExtensionTwo.jar", "ExtensionThree.jar");
		test_render_default(registry, "test.tools");

		write("++Testing extensions " + (ok ? "OK" : "Failed") + "\n");
		return ok;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_item(int i, ExtensionPoint<?> ep)
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+ep.getId()+"\t"+ep.getActionClass()+"\t"+ep.getDescription() +"\t" +
				ep.getRendererFactory() + "\t" + ep.allowRendererOverride());
		}
	}
	
	static void write_item(int i, Extension<?> ext)
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+ext.getId()+"\t"+ext.getExtends()+"\t"+ext.getActionFactory() + "\t" +
				ext.getRendererFactory());
		}
	}
	
	
	static void test_load_extension_points_from_xml(Registry registry, String xmlFile, int expected)
	{
		try
		{
			InputStream in = new FileInputStream(xmlFile);
			
			XmlLoader loader = new XmlLoader();
			VariableConverter converter = new VariableConverter();
			converter.setVariable("HOME", "/home");
			converter.setVariable("ROOT", "/root");
			PathConverter path = new PathConverter("/root", "/home");
			loader.addValueConverter(path);
			loader.addValueConverter(converter);
			loader.loadXmlFile(in, xmlFile, null, true);
			
			List<ExtensionPoint<Action>> list = loader.getExtensionPoints();
			for (int i = 0; i < list.size(); ++i)
			{
				ExtensionPoint<Action> ep = list.get(i);
				registry.registerExtensionPoint(ep, null);
				write_item(i, ep);
			}
			if (expected >= 0 && expected != list.size())
			{
				throw new BaseException("Expected "+expected+" extensions, not " + list.size());
			}			
			write("--Load extension points from xml OK (" + list.size() + ")");
		}
		catch (Throwable t)
		{
			write("--Load extension points from xml FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	static void test_load_extensions_from_xml(Registry registry, String xmlFile, int expected)
	{
		try
		{
			InputStream in = new FileInputStream(xmlFile);
			
			XmlLoader loader = new XmlLoader();
			VariableConverter converter = new VariableConverter();
			converter.setVariable("HOME", "/home");
			converter.setVariable("ROOT", "/root");
			PathConverter path = new PathConverter("/root", "/home");
			loader.addValueConverter(path);
			loader.addValueConverter(converter);
			loader.loadXmlFile(in, xmlFile, null, true);
			
			List<Extension<Action>> list = loader.getExtensions();
			for (int i = 0; i < list.size(); ++i)
			{
				Extension<Action> ext = list.get(i);
				registry.registerExtension(ext, null);
				write_item(i, ext);
			}
			if (expected >= 0 && expected != list.size())
			{
				throw new BaseException("Expected "+expected+" extensions, not " + list.size());
			}
			
			write("--Load extensions from xml OK (" + list.size() + ")");
		}
		catch (Throwable t)
		{
			write("--Load extensions from xml FAILED");
			t.printStackTrace();
			ok = false;
		}
	}
	
	static void test_load_extensions_from_jar(ExtensionsManager manager, int expected, String... jarFiles)
	{
		try
		{
			for (String jarFile : jarFiles)
			{
				manager.addFile(new File(jarFile));
			}
			
			RegisterExtensionsProcessor register = new RegisterExtensionsProcessor(new XmlLoader(), null);
			manager.processFiles(register);
			
			int numRegistered = register.getNumRegistered();
			if (expected >= 0 && expected != numRegistered)
			{
				throw new BaseException("Expected "+expected+" extensions, not " + numRegistered);
			}
			
			write("--Load extensions from JAR OK (" + numRegistered + ")");
		}
		catch (Throwable t)
		{
			write("--Register extensions from JAR FAILED");
			t.printStackTrace();
			ok = false;
		}
		
	}
	
	@SuppressWarnings("unchecked")
	static void test_render_default(Registry registry, String extensionPoint)
	{
		try
		{
			ClientContext clientContext = new ClientContext(TestUtil.getSessionControl());
			ExtensionsInvoker<ActionButton> manager = 
				(ExtensionsInvoker<ActionButton>)registry.useExtensions(clientContext, null, extensionPoint);
			
			manager.renderDefault();
			write("--NOTE!! There should be 4 actions, 1 error message and then 3 more actions above");
			write("--Render default OK");
		}
		catch (Throwable t)
		{
			write("--Render default FAILED");
			t.printStackTrace();
			ok = false;
		}
		
	}
	
	public static class ActionButton
		implements IActionButton
	{
		private String title;
		private String icon;
		private int type;
		private boolean disabled;
		
		public ActionButton()
		{}
		
		public ActionButton(String title, String icon, int type, boolean disabled)
		{
			this.title = title;
			this.icon = icon;
			this.type = type;
			this.disabled = disabled;
		}
		
		@Override
		public String getTitle()
		{
			return title;
		}
		
		public void setTitle(String title)
		{
			this.title = title;
		}
		
		@Override
		public String getIcon()
		{
			return icon;
		}
		
		public void setIcon(String icon)
		{
			this.icon = icon;
		}
	
		@Override
		public int getType()
		{
			return type;
		}
		
		public void setType(int type)
		{
			this.type = type;
		}
		
		@Override
		public boolean isDisabled()
		{
			return disabled;
		}
		
		public void setDisabled(boolean disabled)
		{
			this.disabled = disabled;
		}
	}

	public static class ActionButtonFactory
		implements ActionFactory<IActionButton>
	{

		private String title;
		private String icon;
		private int type;
		private boolean disabled;
		
		private IActionButton button;
		
		public ActionButtonFactory()
		{}
		
		@Override
		public IActionButton[] getActions(InvokationContext<? super IActionButton> context)
		{
			if (button == null)
			{
				button = new ActionButton(title, icon, type, disabled);
			}
			return new IActionButton[] { button };
		}

		@Override
		public boolean prepareContext(InvokationContext<? super IActionButton> context) 
		{
			return true;
		}
		
		public void setTitle(String title)
		{
			this.title = title;
		}
		
		@VariableSetter
		@PathSetter
		public void setIcon(String icon)
		{
			this.icon = icon;
		}
		
		public void setType(String type)
		{
			this.type = Values.getInt(type);
		}
		
		public void setDisabled(String disabled)
		{
			this.disabled = Values.getBoolean(disabled);
		}
				
	}
	
	public static class InvalidActionFactory
		implements ActionFactory<Action>
	{
	
		public InvalidActionFactory()
		{}
		
		@Override
		public Action[] getActions(InvokationContext<? super Action> context)
		{
			return new Action[] { new InvalidAction() };
		}
	
		@Override
		public boolean prepareContext(InvokationContext<? super Action> context) 
		{
			return true;
		}				
	}

	
	public static class InvalidAction
		implements Action
	{}
	
	public static class ActionButtonRendererFactory
		implements RendererFactory<IActionButton>, Renderer<IActionButton>
	{
		
		private String color = "#ffffff";
		
		public ActionButtonRendererFactory()
		{}
		
		public void setColor(String color)
		{
			this.color = color;
		}

		@Override
		public void prepareContext(InvokationContext<? extends IActionButton> context) 
		{
		}
		
		@Override
		public Renderer<IActionButton> getRenderer(InvokationContext<? extends IActionButton> context) 
		{
			return this;
		}

		@Override
		public void render(IActionButton action) 
		{
			if (!TestUtil.getSilent() && !action.isDisabled())
			{
				System.out.println("<div class=\"actionbutton\" style=\"color: " + 
						color + ";\"><img src=\"" + action.getIcon() + "\">" + 
						action.getTitle() + " (type=" + action.getType() + ")</div>");
			}
		}
	}
	
}
